package com.hiramine.modelfileloader;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.util.StringTokenizer;

import android.util.Log;

public class MtlFileLoader
{
	private static final String	NONAME	= "(NoName)";

	public static MFLMaterial[] load( String strPath )
	{
		File file = new File( strPath );
		long lFileSize = file.length();
		if( 0 == lFileSize )
		{
			return null;
		}
		try
		{
			BufferedReader br = new BufferedReader( new FileReader( strPath ) );

			br.mark( (int)lFileSize );

			// ファーストパース（要素数カウント）
			int[] aiCountMaterial = new int[1];
			if( !parse_first( br, aiCountMaterial ) )
			{
				return null;
			}

			// 領域確保
			if( 0 == aiCountMaterial[0] )
			{
				return null;
			}
			MFLMaterial[] aMaterial = new MFLMaterial[aiCountMaterial[0]];
			for( int i = 0; i < aMaterial.length; ++i )
			{
				aMaterial[i] = new MFLMaterial();
			}
			aMaterial[0].strName = NONAME;

			br.reset();

			// セカンドパース（値詰め）
			if( !parse_second( br, aMaterial ) )
			{
				return null;
			}

			br.close();

			return aMaterial;
		}
		catch( Exception e )
		{
			Log.e( "MtlFileLoader", "load error : " + e );
		}
		return null;
	}

	private static boolean parse_first( BufferedReader br, int[] aiCountMaterial )
	{
		// インプットのチェック
		if( null == aiCountMaterial )
		{
			return false;
		}

		// アウトプットの初期化
		aiCountMaterial[0] = 0;

		// 読み取り
		int iCountMaterial = 1; // (NoName)分
		while( true )
		{
			try
			{
				String strReadString = br.readLine();
				if( null == strReadString )
				{
					break;
				}
				StringTokenizer stReadString = new StringTokenizer( strReadString, ", \t\r\n" );
				if( !stReadString.hasMoreTokens() )
				{
					continue;
				}
				String token = stReadString.nextToken();
				switch( token.charAt( 0 ) )
				{
				case 'n': // material
					++iCountMaterial;
					break;
				}
			}
			catch( Exception e )
			{
				Log.e( "MtlFileLoader", "parse_first error : " + e );
				return false;
			}
		}
		aiCountMaterial[0] = iCountMaterial;
		return true;
	}

	private static boolean parse_second( BufferedReader br, MFLMaterial[] aMaterial )
	{
		// インプットのチェック
		if( null == aMaterial )
		{
			return false;
		}

		// 読み取り
		float f0;
		float f1;
		float f2;
		int iIndexMaterial = 0;
		MFLMaterial material_current = aMaterial[iIndexMaterial];
		while( true )
		{
			try
			{
				String strReadString = br.readLine();
				if( null == strReadString )
				{
					break;
				}
				StringTokenizer stReadString = new StringTokenizer( strReadString, ", \t\r\n" );
				if( !stReadString.hasMoreTokens() )
				{
					continue;
				}
				String token = stReadString.nextToken();
				switch( token.charAt( 0 ) )
				{
				case '#': // コメント
					break;
				case 'n': // マテリアル
					++iIndexMaterial;
					material_current = aMaterial[iIndexMaterial];
					if( stReadString.hasMoreTokens() )
					{
						material_current.strName = stReadString.nextToken();
					}
					else
					{
						material_current.strName = NONAME;
					}
					break;
				case 'N':
					if( stReadString.hasMoreTokens() )
					{
						material_current.fShininess = Float.parseFloat( stReadString.nextToken() );
					}
					/* wavefrontフォーマットのshininessは[0, 1000]、OpenGLのshininessは[0, 128] */
					material_current.fShininess /= 1000.0;
					material_current.fShininess *= 128.0;
					break;
				case 'K':
					f0 = 0.0f;
					f1 = 0.0f;
					f2 = 0.0f;
					if( stReadString.hasMoreTokens() )
					{
						f0 = Float.parseFloat( stReadString.nextToken() );
					}
					if( stReadString.hasMoreTokens() )
					{
						f1 = Float.parseFloat( stReadString.nextToken() );
					}
					if( stReadString.hasMoreTokens() )
					{
						f2 = Float.parseFloat( stReadString.nextToken() );
					}
					switch( token.charAt( 1 ) )
					{
					case 'a':
						material_current.f4Ambient[0] = f0;
						material_current.f4Ambient[1] = f1;
						material_current.f4Ambient[2] = f2;
						break;
					case 'd':
						material_current.f4Diffuse[0] = f0;
						material_current.f4Diffuse[1] = f1;
						material_current.f4Diffuse[2] = f2;
						break;
					case 's':
						material_current.f4Specular[0] = f0;
						material_current.f4Specular[1] = f1;
						material_current.f4Specular[2] = f2;
						break;
					default:
						break;
					}
					break;
				default:
					break;
				}
			}
			catch( Exception e )
			{
				Log.e( "MtlFileLoader", "parse_second" );
				return false;
			}
		}
		return true;
	}
}
