package com.hiramine.modelvieweretude;

import android.app.ProgressDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.DialogInterface.OnCancelListener;
import android.content.DialogInterface.OnClickListener;
import android.os.AsyncTask;

import com.hiramine.modelfileloader.OnProgressListener;

public class ModelFileLoadTask extends AsyncTask<Void, Integer, Void>
		implements OnCancelListener, OnClickListener, OnProgressListener
{

	private ProgressDialog	m_progressdialog;
	private ModelViewerView	m_modelviewerview;
	private String			m_strPath;

	public ModelFileLoadTask( Context context, ModelViewerView modelviewerview, String strPath )
	{
		m_progressdialog = new ProgressDialog( context );
		m_modelviewerview = modelviewerview;
		m_strPath = strPath;
	}

	// タスクの前処理
	@Override
	protected void onPreExecute()
	{
		// プログレスダイアログを表示
		m_progressdialog.setProgressStyle( ProgressDialog.STYLE_HORIZONTAL );
		m_progressdialog.setMax( 100 );
		m_progressdialog.setTitle( "Load model file" );
		m_progressdialog.setMessage( m_strPath );
		m_progressdialog.setCancelable( true );
		m_progressdialog.setOnCancelListener( this );
		m_progressdialog.setButton( "Cancel", this );
		m_progressdialog.show();
	}

	// タスク
	@Override
	protected Void doInBackground( Void... params )
	{
		m_modelviewerview.loadModelFile( m_strPath, this );
		return null;
	}

	// 進捗の更新
	@Override
	protected void onProgressUpdate( Integer... progress )
	{
		m_progressdialog.setMax( progress[1].intValue() );
		m_progressdialog.setProgress( progress[0].intValue() );
	}

	// タスクの後処理
	@Override
	protected void onPostExecute( Void result )
	{
		// プログレスダイアログを閉じる
		m_progressdialog.dismiss();
	}

	// タスクがキャンセルされたときの処理
	@Override
	protected void onCancelled()
	{
		// プログレスダイアログを閉じる
		m_progressdialog.dismiss();
	}

	// 進捗ダイアログがキャンセルされた
	public void onCancel( DialogInterface arg0 )
	{
		this.cancel( true ); // タスクのキャンセル要請
	}

	// 「キャンセルボタン」ボタンが押された
	public void onClick( DialogInterface arg0, int arg1 )
	{
		m_progressdialog.cancel(); // 進捗ダイアログのキャンセル要請
	}

	// 進捗の監視対象の進捗が更新された
	public void onProgress( int iPos, int iMax )
	{
		publishProgress( iPos, iMax ); // onProgressUpdateに情報をトス
	}
}
